@@ -13,13 +13,17 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import imp
+import atexit
+try:
+    import _imp as imp
+except ImportError:
+    import imp
 import os
 import sys
 import traceback
 
+import eventlet
 from eventlet import event, greenio, greenthread, patcher, timeout
-from eventlet.support import six
 
 __all__ = ['execute', 'Proxy', 'killall', 'set_num_threads']
 
@@ -31,15 +35,12 @@ QUIET = True
 
 socket = patcher.original('socket')
 threading = patcher.original('threading')
-if six.PY2:
-    Queue_module = patcher.original('Queue')
-if six.PY3:
-    Queue_module = patcher.original('queue')
+Queue_module = patcher.original('queue')
 
 Empty = Queue_module.Empty
 Queue = Queue_module.Queue
 
-_bytetosend = ' '.encode()
+_bytetosend = b' '
 _coro = None
 _nthreads = int(os.environ.get('EVENTLET_THREADPOOL_SIZE', 20))
 _reqq = _rspq = None
@@ -54,6 +55,7 @@ def tpool_trampoline():
         try:
             _c = _rsock.recv(1)
             assert _c
+        # FIXME: this is probably redundant since using sockets instead of pipe now
         except ValueError:
             break  # will be raised when pipe is closed
         while not _rspq.empty():
@@ -82,6 +84,7 @@ def tworker():
             raise
         except EXC_CLASSES:
             rv = sys.exc_info()
+            traceback.clear_frames(rv[1].__traceback__)
         # test_leakage_from_tracebacks verifies that the use of
         # exc_info does not lead to memory leaks
         _rspq.put((e, rv))
@@ -104,7 +107,7 @@ def execute(meth, *args, **kwargs):
     # if already in tpool, don't recurse into the tpool
     # also, call functions directly if we're inside an import lock, because
     # if meth does any importing (sadly common), it will hang
-    my_thread = threading.currentThread()
+    my_thread = threading.current_thread()
     if my_thread in _threads or imp.lock_held() or _nthreads == 0:
         return meth(*args, **kwargs)
 
@@ -119,7 +122,7 @@ def execute(meth, *args, **kwargs):
         if not QUIET:
             traceback.print_exception(c, e, tb)
             traceback.print_stack()
-        six.reraise(c, e, tb)
+        raise e.with_traceback(tb)
     return rv
 
 
@@ -145,7 +148,7 @@ def proxy_call(autowrap, f, *args, **kwargs):
         return rv
 
 
-class Proxy(object):
+class Proxy:
     """
     a simple proxy-wrapper of any object that comes with a
     methods-only interface, in order to forward every method
@@ -250,54 +253,80 @@ class Proxy(object):
 
 
 def setup():
-    global _rsock, _wsock, _threads, _coro, _setup_already, _rspq, _reqq
+    global _rsock, _wsock, _coro, _setup_already, _rspq, _reqq
     if _setup_already:
         return
     else:
         _setup_already = True
 
-    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-    sock.bind(('127.0.0.1', 0))
-    sock.listen(1)
-    csock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-    csock.connect(sock.getsockname())
-    _wsock, _addr = sock.accept()
-    sock.close()
-    _rsock = greenio.GreenSocket(csock)
-
-    _reqq = Queue(maxsize=-1)
-    _rspq = Queue(maxsize=-1)
     assert _nthreads >= 0, "Can't specify negative number of threads"
     if _nthreads == 0:
         import warnings
         warnings.warn("Zero threads in tpool.  All tpool.execute calls will\
             execute in main thread.  Check the value of the environment \
             variable EVENTLET_THREADPOOL_SIZE.", RuntimeWarning)
-    for i in six.moves.range(_nthreads):
+    _reqq = Queue(maxsize=-1)
+    _rspq = Queue(maxsize=-1)
+
+    # connected socket pair
+    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+    sock.bind(('127.0.0.1', 0))
+    sock.listen(1)
+    csock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+    csock.connect(sock.getsockname())
+    csock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, True)
+    _wsock, _addr = sock.accept()
+    _wsock.settimeout(None)
+    _wsock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, True)
+    sock.close()
+    _rsock = greenio.GreenSocket(csock)
+    _rsock.settimeout(None)
+
+    for i in range(_nthreads):
         t = threading.Thread(target=tworker,
                              name="tpool_thread_%s" % i)
-        t.setDaemon(True)
+        t.daemon = True
         t.start()
         _threads.append(t)
 
     _coro = greenthread.spawn_n(tpool_trampoline)
+    # This yield fixes subtle error with GreenSocket.__del__
+    eventlet.sleep(0)
 
 
+# Avoid ResourceWarning unclosed socket on Python3.2+
+@atexit.register
 def killall():
     global _setup_already, _rspq, _rsock, _wsock
     if not _setup_already:
         return
+
+    # This yield fixes freeze in some scenarios
+    eventlet.sleep(0)
+
     for thr in _threads:
         _reqq.put(None)
     for thr in _threads:
         thr.join()
     del _threads[:]
+
+    # return any remaining results
+    while (_rspq is not None) and not _rspq.empty():
+        try:
+            (e, rv) = _rspq.get(block=False)
+            e.send(rv)
+            e = rv = None
+        except Empty:
+            pass
+
     if _coro is not None:
         greenthread.kill(_coro)
-    _rsock.close()
-    _wsock.close()
-    _rsock = None
-    _wsock = None
+    if _rsock is not None:
+        _rsock.close()
+        _rsock = None
+    if _wsock is not None:
+        _wsock.close()
+        _wsock = None
     _rspq = None
     _setup_already = False
 
